import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import EventEmitter from 'events';
import createPage from './page';

const createBrowser = send => {
  const eventEmitter = new EventEmitter();
  return {
    on: (event, callback) => {
      eventEmitter.on(event, callback);
    },
    once: (event, callback) => {
      eventEmitter.once(event, callback);
    },
    off: (event, callback) => {
      eventEmitter.off(event, callback);
    },
    close: function () {
      var _close = _asyncToGenerator(function* () {
        yield send('Marionette:AcceptConnections', {
          value: false
        });
        yield send('Marionette:Quit');
        eventEmitter.emit('disconnected');
      });

      return function close() {
        return _close.apply(this, arguments);
      };
    }(),
    disconnect: function () {
      var _disconnect = _asyncToGenerator(function* () {
        yield send('WebDriver:DeleteSession');
        eventEmitter.emit('disconnected');
      });

      return function disconnect() {
        return _disconnect.apply(this, arguments);
      };
    }(),
    newPage: function () {
      var _newPage = _asyncToGenerator(function* () {
        yield send('WebDriver:ExecuteScript', {
          script: 'window.open()'
        });
        const pages = yield send('WebDriver:GetWindowHandles');
        return createPage(send, pages[pages.length - 1]);
      });

      return function newPage() {
        return _newPage.apply(this, arguments);
      };
    }(),
    pages: function () {
      var _pages = _asyncToGenerator(function* () {
        const ids = yield send('WebDriver:GetWindowHandles');
        return ids.map(id => createPage(send, id));
      });

      return function pages() {
        return _pages.apply(this, arguments);
      };
    }()
  };
};

export default createBrowser;