import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";

/* eslint-disable no-use-before-define */
import { writeFile } from 'fs';
import makethen from 'makethen';
import createElement from './element';
const pWriteFile = makethen(writeFile);

const createPage = (send, id) => {
  const switchToPage = () => send('WebDriver:SwitchToWindow', {
    name: id
  });

  return {
    $: function () {
      var _$ = _asyncToGenerator(function* (selector) {
        yield switchToPage();

        const _ref = yield send('WebDriver:FindElement', {
          value: selector,
          using: 'css selector'
        }),
              value = _ref.value;

        return createElement(send, value.ELEMENT);
      });

      return function $(_x) {
        return _$.apply(this, arguments);
      };
    }(),
    $$: function () {
      var _$$ = _asyncToGenerator(function* (selector) {
        yield switchToPage();
        const values = yield send('WebDriver:FindElements', {
          value: selector,
          using: 'css selector'
        });
        return values.map(value => createElement(send, value.ELEMENT));
      });

      return function $$(_x2) {
        return _$$.apply(this, arguments);
      };
    }(),
    close: function () {
      var _close = _asyncToGenerator(function* () {
        yield switchToPage();
        yield send('WebDriver:ExecuteScript', {
          script: 'window.close()'
        });
      });

      return function close() {
        return _close.apply(this, arguments);
      };
    }(),
    evaluate: function () {
      var _evaluate = _asyncToGenerator(function* (target, ...args) {
        yield switchToPage();

        if (typeof target === 'function') {
          const _ref2 = yield send('WebDriver:ExecuteAsyncScript', {
            script: `
          const args = Array.prototype.slice.call(arguments, 0, arguments.length - 1)
          const resolve = arguments[arguments.length - 1]
          const result = (${target.toString()})(args)

          Promise.resolve(result)
            .then(resolve)
            .catch((error) => {
              throw error
            })
        `,
            args
          }),
                value = _ref2.value;

          return value;
        }

        const _ref3 = yield send('WebDriver:ExecuteAsyncScript', {
          script: `
          Promise.resolve(${target})
            .then(arguments[0])
            .catch((error) => {
              throw error
            })
        `
        }),
              value = _ref3.value;

        return value;
      });

      return function evaluate(_x3) {
        return _evaluate.apply(this, arguments);
      };
    }(),
    goto: function () {
      var _goto = _asyncToGenerator(function* (url) {
        yield switchToPage();
        yield send('WebDriver:Navigate', {
          url
        });
      });

      return function goto(_x4) {
        return _goto.apply(this, arguments);
      };
    }(),
    screenshot: function () {
      var _screenshot = _asyncToGenerator(function* (options = {}) {
        yield switchToPage();
        const result = yield send('WebDriver:TakeScreenshot', {
          full: true,
          hash: false
        });
        const buffer = Buffer.from(result.value, 'base64');

        if (typeof options.path === 'string') {
          yield pWriteFile(options.path, buffer);
        }

        return buffer;
      });

      return function screenshot() {
        return _screenshot.apply(this, arguments);
      };
    }(),
    setContent: function () {
      var _setContent = _asyncToGenerator(function* (html) {
        yield switchToPage();
        return send('WebDriver:ExecuteScript', {
          script: 'document.documentElement.innerHTML = arguments[0]',
          args: [html]
        });
      });

      return function setContent(_x5) {
        return _setContent.apply(this, arguments);
      };
    }(),
    title: function () {
      var _title = _asyncToGenerator(function* () {
        yield switchToPage();
        const result = yield send('WebDriver:GetTitle');
        return result.value;
      });

      return function title() {
        return _title.apply(this, arguments);
      };
    }()
  };
};

export default createPage;