import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import { Socket } from 'net';
import { createParseStream, parse, stringify } from './transport';
const CONNECTION_TIMEOUT = 10000;

const connectToMarionette =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* (host, port) {
    let globalId = 0;
    let queue = [];
    const socket = new Socket();
    yield new Promise((resolve, reject) => {
      const rejectAndDestroy = error => {
        reject(error);
        socket.destroy();
      };

      socket.setTimeout(CONNECTION_TIMEOUT).once('connect', () => {
        socket.once('data', rawData => {
          const data = parse(rawData);

          if (data.applicationType === 'gecko') {
            if (data.marionetteProtocol === 3) {
              return resolve();
            }

            return rejectAndDestroy(new Error('Foxr works only with Marionette protocol v3'));
          }

          rejectAndDestroy(new Error('Marionette connection protocol error'));
        });
      }).once('timeout', () => rejectAndDestroy(new Error('Socket connection timeout'))).once('error', rejectAndDestroy).connect(port, host);
    });
    const parseStream = createParseStream();
    parseStream.on('data', data => {
      const _data = _slicedToArray(data, 4),
            type = _data[0],
            id = _data[1],
            error = _data[2],
            result = _data[3];

      if (type === 1) {
        queue = queue.filter(item => {
          if (item.id === id) {
            if (error !== null) {
              item.reject(error);
            } else {
              item.resolve(result);
            }

            return false;
          }

          return true;
        });
      }
    });
    socket.pipe(parseStream);
    return {
      disconnect: () => {
        socket.end();
      },
      send: (name, params = {}) => {
        return new Promise((resolve, reject) => {
          const data = stringify([0, globalId, name, params]);
          socket.write(data, 'utf8', () => {
            queue.push({
              id: globalId,
              resolve,
              reject
            });
            globalId += 1;
          });
        });
      }
    };
  });

  return function connectToMarionette(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

export default connectToMarionette;